#include <propeller.h>
#include "kernel.h"
#include "motor.h"
#include "analog_out.h"
#include "max147.h"
#include "lcd.h"

// the STATIC HUB mailbox for communication to PASM routine 
// a pointer to this gets passed to the PASM code as the PAR register
extern unsigned int value; 

///////////////////////////////////////////////////////////
void msleep(int t)
{
    waitcnt((CLKFREQ/1000)*t+CNT);
}

///////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////

int start(unsigned int *value)
{
    // The label binary_sigma_delta_dat_start is automatically placed
    // on the cog code from sigme_delta.dat by objcopy (see the Makefile).
    extern unsigned int binary_sigma_delta_dat_start[];
    
    return cognew(&binary_sigma_delta_dat_start, value);
}



///////////////////////////////////////////////////////////

void start_lmm(void *func, int *stack_top)
{
    extern unsigned int _load_start_kernel[];

    /* push the code address onto the stack */
    --stack_top;
    *stack_top = (int)func;

    /* now start the kernel */
    cognew(_load_start_kernel, stack_top);
}


///////////////////////////////////////////////////////////
void Task_PWM_Pin7 ()
{
	int t;
	int tInc = CLKFREQ/1000000;
	int tC = 100 * tInc;

	CTRA = (1<<28) + 7;
	FRQA = 1;
	DIRA |= (1 << 7);
	
	t = CNT;
	
	while(1)
	{
		PHSA = - MotorControl.motor_A_power * tInc;
		t += tC;
		if(INA & (1 << Button2))
			ButtonEvent.Button2State = CLOSE;
		waitcnt(t);
	}
	
	

}

///////////////////////////////////////////////////////////
void Task_PWM_Pin6 ()
{
	int t;
	int tInc = CLKFREQ/1000000;
	int tC = 100 * tInc;

	CTRA = (1<<28) + 6;
	FRQA = 1;
	DIRA |= (1 << 6);
	
	t = CNT;
	
	while(1)
	{
		PHSA = - MotorControl.motor_B_power * tInc;
		t += tC;
		if(INA & (1 << Button1))
			ButtonEvent.Button1State = CLOSE;		
		waitcnt(t);
	}
	
	

}

///////////////////////////////////////////////////////////
void Task_Duty_Pin4()
{
	DIRA |= ANALOG_OUT;			// P3 = OUTPUT
	CTRA = CRTMODE;				// set Counter Mode: DUTY SINGLE ENDED
	while(1)
		FRQA = AOUT;
}

///////////////////////////////////////////////////////////


void kernel_init ()
{
	
	// Sigma Delta ADC
	// start a new COG passing a pointer to the STATIC mailbox structure
 	start(&value);
	
		
	// motor
	MotorControl.motor_A_power = 0;
	MotorControl.motor_B_power = 0;
	MotorControl.motor_A_direction = fwd;
	MotorControl.motor_B_direction = fwd;
	
	start_lmm(Task_PWM_Pin7, cog_stack + STACKSIZE);
	start_lmm(Task_PWM_Pin6, cog_stack + 2 * STACKSIZE);
	
	DIRA |= Motor_A_Dir;
	DIRA |= Motor_B_Dir;
	
	// analog out
	start_lmm(Task_Duty_Pin4, cog_stack + 3 * STACKSIZE);
	
	//Buttons
	ButtonEvent.Button1State = OPEN;
	ButtonEvent.Button2State = OPEN;
	
	//adc_fast
	initMAX147();
	
	//lcd
	lcd_init();

	
	
}

///////////////////////////////////////////////////////////
unsigned char getButtonState (unsigned char button)
{
	if(button == 1)
	{
		if(ButtonEvent.Button1State == CLOSE)
		{
			ButtonEvent.Button1State = OPEN;
			return CLOSE;
		}
		else
			return OPEN;
	}
	
	if(button == 2)
	{
		if(ButtonEvent.Button2State == CLOSE)
		{
			ButtonEvent.Button2State = OPEN;
			return CLOSE;
		}
		else
			return OPEN;
	}
	return 0xFF; // undef
}
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////